package sun.swing.table;

import java.io.Serializable;
import java.awt.Insets;
import java.awt.FontMetrics;
import javax.swing.SwingUtilities;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.Graphics;
import java.util.List;
import javax.swing.RowSorter;
import javax.swing.border.Border;
import javax.swing.SortOrder;
import java.awt.Color;
import javax.swing.table.JTableHeader;
import sun.swing.DefaultLookup;
import java.awt.Component;
import javax.swing.JTable;
import javax.swing.Icon;
import javax.swing.plaf.UIResource;
import javax.swing.table.DefaultTableCellRenderer;

public class DefaultTableCellHeaderRenderer extends DefaultTableCellRenderer implements UIResource
{
    private boolean horizontalTextPositionSet;
    private Icon sortArrow;
    private DefaultTableCellHeaderRenderer.EmptyIcon emptyIcon;
    
    public DefaultTableCellHeaderRenderer() {
        this.emptyIcon = new DefaultTableCellHeaderRenderer.EmptyIcon();
        this.setHorizontalAlignment(0);
    }
    
    @Override
    public void setHorizontalTextPosition(int horizontalTextPosition) {
        this.horizontalTextPositionSet = true;
        super.setHorizontalTextPosition(horizontalTextPosition);
    }
    
    @Override
    public Component getTableCellRendererComponent(JTable table, Object o, boolean b, boolean b2, int n, int n2) {
        Icon icon = null;
        boolean paintingForPrint = false;
        if (table != null) {
            JTableHeader tableHeader = table.getTableHeader();
            if (tableHeader != null) {
                Color foreground = null;
                Color background = null;
                if (b2) {
                    foreground = DefaultLookup.getColor(this, this.ui, "TableHeader.focusCellForeground");
                    background = DefaultLookup.getColor(this, this.ui, "TableHeader.focusCellBackground");
                }
                if (foreground == null) {
                    foreground = tableHeader.getForeground();
                }
                if (background == null) {
                    background = tableHeader.getBackground();
                }
                this.setForeground(foreground);
                this.setBackground(background);
                this.setFont(tableHeader.getFont());
                paintingForPrint = tableHeader.isPaintingForPrint();
            }
            if (paintingForPrint == 0 && table.getRowSorter() != null) {
                if (!this.horizontalTextPositionSet) {
                    this.setHorizontalTextPosition(10);
                }
                SortOrder columnSortOrder = getColumnSortOrder(table, n2);
                if (columnSortOrder != null) {
                    switch (columnSortOrder) {
                        case ASCENDING: {
                            icon = DefaultLookup.getIcon(this, this.ui, "Table.ascendingSortIcon");
                            break;
                        }
                        case DESCENDING: {
                            icon = DefaultLookup.getIcon(this, this.ui, "Table.descendingSortIcon");
                            break;
                        }
                        case UNSORTED: {
                            icon = DefaultLookup.getIcon(this, this.ui, "Table.naturalSortIcon");
                            break;
                        }
                    }
                }
            }
        }
        this.setText((o == null) ? "" : o.toString());
        this.setIcon(icon);
        this.sortArrow = icon;
        Border border = null;
        if (b2) {
            border = DefaultLookup.getBorder(this, this.ui, "TableHeader.focusCellBorder");
        }
        if (border == null) {
            border = DefaultLookup.getBorder(this, this.ui, "TableHeader.cellBorder");
        }
        this.setBorder(border);
        return this;
    }
    
    public static SortOrder getColumnSortOrder(JTable table, int viewColumnIndex) {
        SortOrder sortOrder = null;
        if (table == null || table.getRowSorter() == null) {
            return sortOrder;
        }
        List<Object> sortKeys = table.getRowSorter().getSortKeys();
        if (sortKeys.size() > 0 && ((RowSorter.SortKey)sortKeys.get(0)).getColumn() == table.convertColumnIndexToModel(viewColumnIndex)) {
            sortOrder = ((RowSorter.SortKey)sortKeys.get(0)).getSortOrder();
            return sortOrder;
        }
        return sortOrder;
    }
    
    public void paintComponent(Graphics graphics) {
        if (DefaultLookup.getBoolean(this, this.ui, "TableHeader.rightAlignSortArrow", false) && this.sortArrow != null) {
            this.emptyIcon.width = this.sortArrow.getIconWidth();
            this.emptyIcon.height = this.sortArrow.getIconHeight();
            this.setIcon(this.emptyIcon);
            super.paintComponent(graphics);
            Point computeIconPosition = this.computeIconPosition(graphics);
            this.sortArrow.paintIcon(this, graphics, computeIconPosition.x, computeIconPosition.y);
        }
        else {
            super.paintComponent(graphics);
        }
    }
    
    private Point computeIconPosition(Graphics graphics) {
        FontMetrics fontMetrics = graphics.getFontMetrics();
        Rectangle viewR = new Rectangle();
        Rectangle textR = new Rectangle();
        Rectangle iconR = new Rectangle();
        Insets insets = this.getInsets();
        viewR.x = insets.left;
        viewR.y = insets.top;
        viewR.width = this.getWidth() - (insets.left + insets.right);
        viewR.height = this.getHeight() - (insets.top + insets.bottom);
        SwingUtilities.layoutCompoundLabel(this, fontMetrics, this.getText(), this.sortArrow, this.getVerticalAlignment(), this.getHorizontalAlignment(), this.getVerticalTextPosition(), this.getHorizontalTextPosition(), viewR, iconR, textR, this.getIconTextGap());
        return new Point(this.getWidth() - insets.right - this.sortArrow.getIconWidth(), iconR.y);
    }
    
    private class EmptyIcon implements Icon, Serializable
    {
        int width;
        int height;
        
        private EmptyIcon() {
            this.width = 0;
            this.height = 0;
        }
        
        @Override
        public void paintIcon(Component component, Graphics graphics, int n, int n2) {
        }
        
        @Override
        public int getIconWidth() {
            return this.width;
        }
        
        @Override
        public int getIconHeight() {
            return this.height;
        }
    }
}
